"""
delimited.mapping
~~~~~~~~~~~~~~~~~
"""


class NestedMapping(object):

    def __iter__(self):
        for key, value in self.data.items():
            yield key, value

    def items(self):
        for key, value in self.data.items():
            yield (key, value)

    def keys(self):
        for key in self.data.keys():
            yield key

    def values(self):
        for value in self.data.values():
            yield value

    def update(self, data, path=None):
        if isinstance(data, self.__class__):
            data = data.unwrap()

        self.ref(path).update(data)



"""
delimited.sequence
~~~~~~~~~~~~~~~~~~
"""

class SequenceIndex(object):
    sequence = None
    
    def __init__(self, index):
        if not isinstance(index, int):
            raise TypeError(index)
        self.index = index
        
    def __str__(self):
        return str(self.index)
        
    def __repr__(self):
        return f'{self.__class__.__name__}({self.index})'
        
    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.index == other.index
        if isinstance(other, int):
            return self.index == other
        return False
        
    def __ne__(self, other):
        return not self.__eq__(other)
        
    def __hash__(self):
        return hash(str(self.index))


class ListIndex(SequenceIndex):
    sequence = list


class SequenceValue(object):
    pass


class NestedSequence(object):
    sequenceindex = None
        
    def __add__(self, other):
        if isinstance(other, self.__class__):
            data = other.data
        elif isinstance(other, self.container):
            data = other
        
        return self.__class__(self.data + data)
        
    def __iadd__(self, other):
        if isinstance(other, self.__class__):
            data = other.data
        elif isinstance(other, self.container):
            data = other
            
        self.data += other
        
        return self
        
    def __iter__(self):
        for v in self.data:
            yield v
        
    def __reversed__(self):
        for v in self.data[::-1]:
            yield v
        
    def append(self, value):
        return self.data.append(value)
        
    def extend(self, other):
        if isinstance(other, self.__class__):
            data = other.data
        elif isinstance(other, self.container):
            data = other
        
        return self.data.extend(data)
        
    def insert(self, i, value):
        return self.data.insert(i, value)
        
    def remove(self, value):
        return self.data.remove(value)
        
    def pop(self, *args):
        return self.data.pop(*args)
        
    def index(self, value):
        return self.data.index(value)
        
    def count(self, value):
        return self.data.count(value)
        
    def reverse(self):
        self.data = self.data[::-1]



import io
import os
import sys
import shutil
import subprocess
import setuptools


# read metadata
setup_path = os.path.abspath(os.path.dirname(__file__))
metadata_path = os.path.join(setup_path, "delimited", "__init__.py")
metadata = {}
with open(metadata_path) as file_handle:
    exec(file_handle.read(), metadata)


setuptools.setup(
    name=metadata["__title__"],
    version=metadata["__version__"],
    description=metadata["__summary__"],
    keywords="delimited nested",
    url=metadata["__url__"],
    author=metadata["__author__"],
    author_email=metadata["__email__"],
    license=metadata["__license__"],
    packages=["delimited"],
    extras_require={
        "test": ["green", "coverage"],
        "docs": ["sphinx", "sphinx_rtd_theme"]
    },
    classifiers=[
        "Development Status :: 3 - Alpha"
    ]
)
